// Copyright 2017 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package ecdsa

import (
	"crypto"
	"crypto/elliptic"
	boring "crypto/internal/backend"
	"crypto/rand"
	"testing"
)

func testHashSignAndHashVerify(t *testing.T, c elliptic.Curve, tag string) {
	priv, err := GenerateKey(c, rand.Reader)
	if priv == nil {
		t.Fatal(err)
	}

	msg := []byte("testing")
	h := crypto.SHA256
	hsm, err := HashSignASN1(rand.Reader, priv, msg, h)
	if err != nil {
		t.Errorf("%s: error signing: %s", tag, err)
		return
	}

	if !HashVerifyASN1(&priv.PublicKey, h, msg, hsm) {
		t.Errorf("%s: Verify failed", tag)
	}

	msg[0] ^= 0xff
	if HashVerifyASN1(&priv.PublicKey, h, msg, hsm) {
		t.Errorf("%s: Verify should not have succeeded", tag)
	}
}

func TestHashSignAndHashVerifyASN1(t *testing.T) {
	testHashSignAndHashVerify(t, elliptic.P256(), "p256")

	if testing.Short() && !boring.Enabled() {
		return
	}
	testHashSignAndHashVerify(t, elliptic.P384(), "p384")
	testHashSignAndHashVerify(t, elliptic.P521(), "p521")
}
